/***************************************************************************
 *   Copyright (C) 2010 by Martin Burgemeister, Mathis Lindemann           *
 *                                                                         *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "UCQBlockList.h"

/**
 * \brief Sets the selection mode and connects the Signal itemSelectionChanged
 *        to selectionC.
 *
 * \param parent Pointer to the parent QWidget.
 */
UCQBlockList::UCQBlockList ( QWidget* parent ) :
    QListWidget ( parent )
{
    setSelectionMode ( QAbstractItemView::ExtendedSelection );
    insertItem ( 0, tr ( "Show all" ) );

    lastSort_ = 0;

    connect ( this, SIGNAL ( itemSelectionChanged () ), this,
              SLOT ( selectionC () )
    );
}

/**
 * \brief Slot for setting new items.
 *
 * The first item will always be the String "Show all" (to show all model
 * items).
 *
 * \param items The new items.
 */
void UCQBlockList::contentsChanged ( QHash<QString, uint> items )
{
    itemsHash_ = items;
    sortBlocks(lastSort_);
 }

/**
 * \brief Slot to (de-)select items.
 *
 * \param names Names of the items to (de-)select.
 * \param select True to select, false to deselect.
 *
 * \see TableController::selectBlocks()
 */
void UCQBlockList::selectBlocks ( QStringList names, bool select )
{
    QList<QListWidgetItem*> list;
    QModelIndex index;
    if ( names.isEmpty () )
    {
        clearSelection ();
    } else
    {
        for ( int i = 0; i < names.size (); ++i )
        {
            // get the items with the correct names
            list = findItems ( names.at ( i ), Qt::MatchExactly );

            if ( !list.isEmpty () )
            {
                // all blocknames are unique, we just select the first hit
                index = indexFromItem ( list.at ( 0 ) );
                selectionModel ()->select (
                                            index,
                                            select
                                                   ? QItemSelectionModel::Select
                                                   : QItemSelectionModel::Deselect );
                scrollTo(index);
            }
        }
    }
}

/**
 * \brief Slot to react on own signal itemSelectionChanged().
 *
 * This Slot is implemented, because itemSelectionChanged contains only the
 * items that were recently added or removed from the selection, but we need
 * all selected items every time.
 * Emits currentBlockChanged and blockChanged.
 */
void UCQBlockList::selectionC ()
{
    QList<QListWidgetItem*> list = selectedItems ();
    QStringList result;

    // get the names of the selected items
    if ( !list.isEmpty () )
    {
        for ( int i = 0; i < list.size (); ++i )
        {
            if ( list.at ( i )->text () != "" )
                result.append ( list.at ( i )->text () );
        }
    }

    // emit the list with the selected blocks
    emit currentBlockChanged ( result );
    emit blockChanged ();
}


/**
 * \brief Sorts the items of the list by the names the blocks.
 *
 * \param ascending If true, the blocks are sorted in ascending order,
 *                  else descending.
 */
void UCQBlockList::sortByName ( bool asceding )
{
    // store selection
    QList<QListWidgetItem*>selected = selectedItems();
    QStringList selectedNames;
    for(int i = 0; i < selected.size(); ++i){
        selectedNames << selected.at(i)->text();
    }

    clear ();
    myItems_.clear ();


    if ( !itemsHash_.isEmpty () )
    {
        QStringList keys = itemsHash_.uniqueKeys ();
        for ( int i = 0; i < keys.size (); ++i )
        {
            // the key is the block name
            QListWidgetItem * temp = new QListWidgetItem ( keys.at ( i ) );

            // first value = lower boundary, second value = upper boundary
            QList<uint> values = itemsHash_.values ( keys.at ( i ) );
            if ( values.size () > 1 )
            {
                temp->setToolTip ( UCTools::formatCodepoint ( values.at ( 0 ) )
                        + " .. " + UCTools::formatCodepoint ( values.at ( 1 ) ) );
            }
            addItem ( temp );
            myItems_.append ( temp );
        }
        sortItems ( asceding ? Qt::AscendingOrder : Qt::DescendingOrder );
    }
    insertItem ( 0, tr ( "Show all" ) );
    selectBlocks(selectedNames,true);
 }

/**
 * \brief Sorts the items of the list by the first codepoints of the blocks.
 *
 * \param ascending If true, the blocks are sorted in ascending order,
 *                  else descending.
 */
void UCQBlockList::sortByCode ( bool ascending )
{
    // store selection
    QList<QListWidgetItem*>selected = selectedItems();
    QStringList selectedNames;
    for(int i = 0; i < selected.size(); ++i){
        selectedNames << selected.at(i)->text();
    }

    clear ();
    QStringList keys = itemsHash_.uniqueKeys ();
    QList<block> blockList;

    for ( int i = 0; i < keys.size (); ++i )
    {
        QList<uint> values = itemsHash_.values ( keys.at ( i ) );
        if ( values.size () > 1 )
        {
            struct block
                    b = { keys.at ( i ), values.at ( 0 ), values.at ( 1 ) };
            blockList.append ( b );
        }
    }
    bool swapped = true;
    int n = blockList.size ();
    while ( n > 0 && swapped )
    {
        swapped = false;
        for ( int i = 0; i < blockList.size () - 1; ++i )
        {
            if ( ascending )
            {
                if ( blockList.at ( i ).start > blockList.at ( i + 1 ).start )
                {
                    blockList.swap ( i, i + 1 );
                    swapped = true;
                }
            } else // (descending)
            {
                if ( blockList.at ( i ).start < blockList.at ( i + 1 ).start )
                {
                    blockList.swap ( i, i + 1 );
                    swapped = true;
                }
            }
        }
        --n;
    }

    for ( int i = 0; i < blockList.size (); ++i )
    {
        QListWidgetItem * temp = new QListWidgetItem ( blockList.at ( i ).name );
        temp->setToolTip ( UCTools::formatCodepoint ( blockList.at ( i ).start )
                + " .. " + UCTools::formatCodepoint ( blockList.at ( i ).end ) );
        addItem ( temp );
        myItems_.append ( temp );
    }

    insertItem ( 0, tr ( "Show all" ) );

    // restore selection
    selectBlocks(selectedNames,true);
}


/**
 * \brief Sorts the blocks of the list in four different ways.
 *
 * \param sorting Defines how the list is sorted with values from 0 to 3.
 */
void UCQBlockList::sortBlocks ( UCQBlockList::SortingOrders order ){
    switch( order ){
        // 0 names ascending
        case SORT_NAME_ASCENDING:
            sortByName ( true );
            break;

        // 1 names descending
        case SORT_NAME_DESCENDING:
            sortByName ( false );
            break;

        // 2 code ascending
        case SORT_CODE_ASCENDING:
            sortByCode ( true );
            break;

        // 3 code descending
        case SORT_CODE_DESCENDING:
            sortByCode ( false );
            break;
    }
    lastSort_ = order;
}
