/***************************************************************************
 *   Copyright (C) 2010 by Martin Burgemeister, Mathis Lindemann           *
 *                                                                         *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef UCQBLOCKLIST_H_
#define UCQBLOCKLIST_H_
#include <QListWidget>
#include "tools/UCTools.h"
#include <QAction>
#include <QDebug>
/**
 * \class UCQBlockList
 * \brief Extended QListWidget for displaying and selecting blocks by name.
 *
 * This widget was written to display the blocks contained in a TableModel.
 * It is synchronized when TableModel emits the blocksChanged() signal and emits
 * the signal currentBlockChanged() when the selection changes, which is received
 * by the TableModel and changes its visible table. For other listeners, that
 * do not care about the selected items, blockChanged() is emitted.
 *
 * \author Martin Burgemeister <Martin.Burgemeister@gmx.de>,
 *         Mathis Lindemann <MathisLindemann@gmx.net>
 * \version 1.0
 * \date 16 August 2010
 *
 */
class UCQBlockList: public QListWidget
{

Q_OBJECT

public:
    UCQBlockList ( QWidget* );

    /**
     * \brief Constants for defining the sorting order.
     *
     * 0 - Blocknames ascending <br>
     * 1 - Blocknames descending <br>
     * 2 - Codepoints ascending <br>
     * 3 - Codepoint desceding  <br>
     */
    enum sortingOrders{
        SORT_NAME_ASCENDING  = 0,
        SORT_NAME_DESCENDING = 1,
        SORT_CODE_ASCENDING  = 2,
        SORT_CODE_DESCENDING = 3
    };

    Q_DECLARE_FLAGS(SortingOrders, sortingOrders)

private:
    QList<QListWidgetItem*> myItems_;
    QHash<QString, uint> itemsHash_;
    UCQBlockList::SortingOrders lastSort_;
    struct block{
        QString name;
        uint start;
        uint end;
    };

public slots:
    void contentsChanged ( QHash<QString, uint> );
    void selectBlocks    ( QStringList, bool );
    void sortByName ( bool );
    void sortByCode ( bool );
    void sortBlocks ( UCQBlockList::SortingOrders );

private slots:
    void selectionC ();

signals:
    /**
     * \brief Signals a change in the selection and provides the selected
     *        blocks.
     *
     * \param blocks The names of the selected blocks.
     */
    void currentBlockChanged ( QStringList blocks);
    /**
     * \brief Signals a change in the selection.
     */
    void blockChanged ();
};

Q_DECLARE_OPERATORS_FOR_FLAGS(UCQBlockList::SortingOrders)

#endif /* UCQBLOCKLIST_H_ */
